﻿/*
 * Written by Stewart Berman 2012
 * Please send any bug reports / suggestions to sab@saberman.com
 * 
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Runtime.InteropServices;
using Wintellect.Interop.Win32API.SQL;
using System.Windows;
using System.Diagnostics;

namespace ODBC
{
    class clsODBC
    {
        clsODBCReferences CODBCReferences = clsODBCReferences.Instance;
        
        Boolean[] bDebugFlags = new bool[256];
        const int DEBUG_FLAGS_MAX = 255;
        const int DEBUG_FLAGS_MIN = 0;

        int nDataSources;
        int nDrivers;

        ArrayList sAttributes = new ArrayList();
        ArrayList sConnectionStrings = new ArrayList();
        ArrayList sDrivers = new ArrayList();
        ArrayList sDescriptions = new ArrayList();
        ArrayList sDriverDescriptions = new ArrayList();
        ArrayList sDSNs = new ArrayList();

        IntPtr henv;
        Boolean benv;

        IntPtr hdbc;
        Boolean bdbc;

        IntPtr hstmt;
        Boolean bstmt;

        const string consTitle = "Information Using ODBC";

        const int connFUNCTION_MAX = 100;
        const int connFUNCTION_USED = 72;

        UInt16[] uFunctions = new UInt16[connFUNCTION_MAX + 1];

        const int connINFORMATION_MAX = 1000;
        const int connINFORMATION_USED = (int)ODBCconst.SQL_INFO_LAST;

        long[] lInformationLong = new long[connINFORMATION_MAX + 1];
        short[] nInformationShort = new short[connINFORMATION_MAX + 1];
        string[] sInformationString = new string[connINFORMATION_MAX + 1];
        string[] sInformationDescription = new string[connINFORMATION_MAX + 1];

        String sConnectDriver;
        String sConnectDSN;
        String sConnectString;
        String sConnectStringOut;

        Boolean bConnect;

        public clsODBC(clsODBCReferences cls)
        {
            //  Save pointer to references class
            CODBCReferences = cls;
            //  Initialize data sources count
            nDataSources = 0;
        }
        ~clsODBC()
        {
            //  Release references
            CODBCReferences = null;
            Delete();
        }
        public void AllocConnect() 
        {
            //  Allocates a connection handle
            try {
                SQLReturnCodes nSQLResults = Win32APISQL.SQLAllocConnect(henv, out hdbc);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                    bdbc = true;
                else
                {
                    CheckError("SQLAllocConnect: ", nSQLResults);
                    bdbc = false;
                }
            }
            catch {
                bdbc = false;
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
        public void AllocEnv()
        {
            //  Allocates an environment handle

            try
            {
                SQLReturnCodes nSQLResults = (SQLReturnCodes)Win32APISQL.SQLAllocEnv(out henv);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                    benv = true;
                else
                {
                    benv = false;
                    CheckError("SQLAllocEnv: ", nSQLResults);
                }
            }
            catch
            {
                benv = false;
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
        public void AllocStmt() 
        {
            //  Allocates a statement handle
            try
            {
                SQLReturnCodes nSQLResults = Win32APISQL.SQLAllocStmt(hdbc, out hstmt);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                    bstmt = true;
                else
                {
                    CheckError("SQLAllocStmt: ", nSQLResults);
                    bstmt = false;
                }
            }
            catch
            {
                bstmt = false;
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
        public string Attributes(int vnAttributes)
        {
            if ((0 > vnAttributes) || (nDrivers <= vnAttributes))
                return "";
            else
                return sAttributes[vnAttributes].ToString();
        }
        public string Attributes(UInt32 vnAttributes)
        {
            if ((1 > vnAttributes) || (nDataSources < vnAttributes))
                return "";
            else
                return sAttributes.GetRange((int)vnAttributes, 1).ToString();
        }
        public void CheckError(string sMessagePrefix, SQLReturnCodes vnSQLResults)
        {
            UInt32 nBuffer;

            UInt32 lSQLResults;

            string sBuffer = "";
            string sErrorMessage;

            StringBuilder sbBuffer = new StringBuilder(512);
            StringBuilder sbSQLState = new StringBuilder(5);

            //  Initialize the error message with the SQL results code
            sErrorMessage = sMessagePrefix + "Error[" + vnSQLResults.ToString("X8") + "]: ";
            //  Get the error information
            try
            {
                SQLReturnCodes nSQLResults = Win32APISQL.SQLError(henv, hdbc, hstmt, sbSQLState, out lSQLResults, sbBuffer, 256, out nBuffer);

                if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                {
                    sErrorMessage = sErrorMessage + "\r\n" + sbBuffer.ToString().Substring(0, (int)nBuffer);
                    // Get additional information about the error
                    while (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults)
                    {
                        nSQLResults = Win32APISQL.SQLError(henv, hdbc, hstmt, sbSQLState, out lSQLResults, sbBuffer, 256, out nBuffer);
                        //Debug.Print("SQLError: " + nSQLResults.ToString());
                        if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                            sErrorMessage = sErrorMessage + sBuffer.Substring(0, (int)nBuffer);
                    }
                }

                //  Tell the user what happened
                MessageBox.Show(sErrorMessage + "\r\n" + "sSQLState[" + sbSQLState.ToString() + "], SQLResults[" + lSQLResults.ToString("X8") + "]", consTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
        public bool Connect(string sDSN, string sUserid, string sPassword)
        {
            //  Connects to the database

            bConnect = false;
            try
            {
                SQLReturnCodes nSQLResults = Win32APISQL.SQLConnect(hdbc, sDSN, SQLUsefull.SQL_NTS, sUserid, SQLUsefull.SQL_NTS, sPassword, SQLUsefull.SQL_NTS);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                {
                    sConnectDSN = sDSN;
                    bConnect = true;
                }
                else
                {
                    CheckError("SQLConnect: ", nSQLResults);
                    bConnect = false;
                }
            }
            catch
            {
                bConnect = false;
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            return bConnect;
        }
        public string ConnectDSN()
        {
            if (bConnect)
                return sConnectDSN;
            else
                return "";
        }
        public string ConnectDriver()
        {
            if (bConnect)
                return sConnectDriver;
            else
                return "";
        }
        public string ConnectString()
        {
            if (bConnect)
                return sConnectString;
            else
                return "";
        }
        public string ConnectStringOut()
        {
            if (bConnect)
                return sConnectStringOut;
            else
                return "";
        }
        public void Delete()
        {
            // Frees allocated items

            if (0 < nDataSources)
            {
                sDSNs.Clear();
                sDSNs.Add("");
                sDescriptions.Clear();
                sDescriptions.Add("");
                nDataSources = 0;
            }
            if (bstmt)
            {
                try
                {
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLFreeStmt(hstmt, (int)SQLUsefull.SQL_DROP);
                    if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                        bstmt = false;
                    else
                        CheckError("SQLFreeStmt: ", nSQLResults);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    // Tell the user what happened
                    MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                    bConnect = false;
                }
            }
            if (bConnect)
            {
                try
                {
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLDisconnect(hdbc);
                    if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                        bConnect = false;
                    else
                        CheckError("SQLDisconnect: ", nSQLResults);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    // Tell the user what happened
                    MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                    bConnect = false;
                }
            }
            if (bdbc)
            {
                try
                {
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLFreeConnect(hdbc);
                    if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                        bdbc = false;
                    else
                        CheckError("SQLFreeConnect: ", nSQLResults);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    // Tell the user what happened
                    MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                    bConnect = false;
                }
            }
            if (benv)
            {
                try
                {
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLFreeEnv(henv);
                    if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                        benv = false;
                    else
                        CheckError("SQLFreeEnv: ", nSQLResults);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    // Tell the user what happened
                    MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                    bConnect = false;
                }
            }
        }
        public bool DriverConnect(string sDriver, string sServer, string sDatabase, string sUserid, string sPassword)
        {
            //  Save the driver
            sConnectDriver = sDriver;

            //  Build Connect String

            sConnectString = "DRIVER={" + sDriver + "}";
            sConnectString = sConnectString + ";SERVER=" + sServer;
            if (sDriver.Contains("Oracle"))
                sConnectString = sConnectString + ";DBQ=" + sServer + ";";
            else
                sConnectString = sConnectString + "SRVR=" + sServer;
            sConnectString = sConnectString + ";DATABASE=" + sDatabase;
            sConnectString = sConnectString + ";UID=" + sUserid;
            sConnectString = sConnectString + ";PWD=" + sPassword;

            //  Connect

            bConnect = false;
            try
            {
                short connectStringOutLenMax = 2047; 
                StringBuilder connectStringOut = new StringBuilder(connectStringOutLenMax + 1);
                short connectStringOutReturn = 0;
                SQLReturnCodes nSQLResults = Win32APISQL.SQLDriverConnect(hdbc, Win32APISQL.GetActiveWindow(), sConnectString, (short)sConnectString.Length, connectStringOut, connectStringOutLenMax, out connectStringOutReturn, (short)SQLDrvCnnct.SQL_DRIVER_NOPROMPT);
                if ((SQLReturnCodes.SQL_SUCCESS == nSQLResults) || (SQLReturnCodes.SQL_SUCCESS_WITH_INFO == nSQLResults))
                {
                    sConnectStringOut = connectStringOut.ToString();
                    bConnect = true;
                }
                else
                {
                    CheckError("SQLDriverConnect: ", nSQLResults);
                    bConnect = false;
                }
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                bConnect = false;
            }
            return bConnect;
        }
        public string Description(int vnDescription)
        {
            if ((0 > vnDescription) || (nDataSources <= vnDescription))
                return "";
            else
                return sDescriptions[vnDescription].ToString();
        }
        public int DataSources()
        {
            //  Saves the DSNs | Descriptions for the ODBC data sources available
            //  Returns the number of ODBC data sources available

            short nDescriptionLen;
            short nDSNLen;
            int nIndex;

            StringBuilder sbDSN = new StringBuilder(32);
            StringBuilder sbDescription = new StringBuilder(128);

            try
            {
                //  Were we called before?
                if (0 == nDataSources)
                {
                    //  Allocate env
                    if (!benv)
                    {
                        AllocEnv();
                        //  Did we get one?
                        if (!benv)
                        {
                            // No -- Give up
                            return nDataSources;
                        }
                    }
                    //  Get the data sources
                    nIndex = 0;
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLDataSources(henv, (short)SQLFetchType.SQL_FETCH_FIRST, sbDSN, 255, out nDSNLen, sbDescription, 255, out nDescriptionLen);
                    while (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                    {
                        nIndex++;
                        sDSNs.Add(sbDSN.ToString(0, nDSNLen));
                        sDescriptions.Add(sbDescription.ToString(0, nDescriptionLen));
                        sbDSN.Clear();
                        sbDescription.Clear();
                        nSQLResults = Win32APISQL.SQLDataSources(henv, (short)SQLFetchType.SQL_FETCH_NEXT, sbDSN, 255, out nDSNLen, sbDescription, 255, out nDescriptionLen);
                    }
                    nDataSources = nIndex;
                }
                return nDataSources;
            }
            catch
            {
                return nDataSources;
            }
        }
        public bool DebugFlags(UInt32 vnDebugFlag)
        {
            if ((DEBUG_FLAGS_MIN <= vnDebugFlag) && (DEBUG_FLAGS_MAX >= vnDebugFlag))
                return bDebugFlags[vnDebugFlag];
            else
                return false;
        }
        public void DebugFlags(UInt32 vnDebugFlag, bool vbDebugFlagValue)
        {
            if ((DEBUG_FLAGS_MIN <= vnDebugFlag) && (DEBUG_FLAGS_MAX >= vnDebugFlag))
                bDebugFlags[vnDebugFlag] = vbDebugFlagValue;
        }
        public int Drivers()
        {
            short nAttributeLen;
            short nDriverLen;
            UInt32 nIndex;

            StringBuilder sbAttributes = new StringBuilder(10240);
            StringBuilder sbDriver = new StringBuilder(256);           
            try
            {
                //  Were we called before:
                if (0 == nDrivers)
                {
                    //  No -- allocate env
                    if (!benv)
                        AllocEnv();
                    //  Did we get one?
                    if (!benv)
                        //  No -- give up
                        return nDrivers;
                    //  Get the drivers
                    nIndex = 0;
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLDrivers(henv, (short)SQLFetchType.SQL_FETCH_FIRST, sbDriver, 255, out nDriverLen, sbAttributes, 10239, out nAttributeLen);
                    while (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                    {
                        //Debug.Print("Drive: " + sbDriver.ToString() + "|" + "Attributes: " + sbAttributes.ToString());
                        nIndex++;
                        sAttributes.Add(sbAttributes.ToString());
                        sDrivers.Add(sbDriver.ToString(0, nDriverLen));
                        sbAttributes.Clear();
                        sbDriver.Clear();
                        nSQLResults = Win32APISQL.SQLDrivers(henv, (short)SQLFetchType.SQL_FETCH_NEXT, sbDriver, 255, out nDriverLen, sbAttributes, 10239, out nAttributeLen);
                    }
                    nDrivers = (int)nIndex;
                }
                return nDrivers;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error in Drivers: " + ex.Message, consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                return nDrivers;
            }
        }
        public string DSN(int vnDSN)
        {
            if ((0>vnDSN) || (nDataSources<=vnDSN))
                return "";
            else
                return sDSNs[vnDSN].ToString();
        }
        public string Driver(int vnDriver)
        {
            if ((0>vnDriver) || (nDrivers<=vnDriver))
                return "";
            else
                return sDrivers[vnDriver].ToString();
        }
        public bool Function(int nIndex)
        {
            if (0 == uFunctions[nIndex])
                return false;
            else
                return true;
        }
        public bool FunctionExists(int nIndex)
        {
            if ((0 >= nIndex) || (connFUNCTION_USED < nIndex))
                return false;
            else
            {
                if (0 != CODBCReferences.nFunctions[nIndex])
                    return true;
                else
                    return false;
            }
        }
        public string FunctionName(int nIndex)
        {
            return CODBCReferences.sFunctions[nIndex];
        }
        public int Functions()
        {
            //  Find out which functions are supported
            GetFunctions((short)ODBCconst.SQL_API_ALL_FUNCTIONS, uFunctions);
            return (int)ODBCconst.SQL_API_ALL_FUNCTIONS;
        }
        public int FunctionsUsed()
        {
            return connFUNCTION_USED;
        }
        public void GetFunctions(short vnFunction, UInt16[] rbFunctions)
        {
            //  Wrapper for the SQLGetFunctiosn API

            try
            {
                for (short nIndex = 1; nIndex <= connFUNCTION_USED; nIndex++)
                {
                    SQLReturnCodes nSQLResults = Win32APISQL.SQLGetFunctions(hdbc, (short)nIndex, out rbFunctions[nIndex]);
                    /*if (SQLReturnCodes.SQL_SUCCESS != nSQLResults)
                        CheckError("SQLGetFunctions: ", nSQLResults)*/
                    ;
                }
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
        public Int32 GetInfoLong(int vfInforType)
        {
            //  Wrapper for the SQLGetInfo API

            UInt32 nBufferlen;
            UInt32 nBufferUsed;

            UInt32 lBuffer;

            if (!bConnect)
                return 0;

            try
            {
                if ((int)SQLDrvCnnct.SQL_DRIVER_HSTMT == vfInforType)
                    lBuffer = (UInt32)hstmt;
                else
                    lBuffer = 0;
                nBufferlen = 4;
                //Debug.Print("SQLGetInfoLong for: " + CODBCReferences.sInformation[CODBCReferences.nInformation[vfInforType]] + ": " + vfInforType);
                SQLReturnCodes nSQLResults = Win32APISQL.SQLGetInfo(hdbc, (short)vfInforType, out lBuffer, nBufferlen, out nBufferUsed);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                {
                    //Debug.Print("SQLGetInfoLong value: " + lBuffer.ToString());
                    return (int)lBuffer;
                }
                else
                {
                    CheckError("SQLGetInfoLong: ", nSQLResults);
                    return 0;
                }
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                return 0;
            }
        }
        public short GetInfoShort(int vfInforType)
        {
            //  Wrapper for the SQLGetInfo API

            UInt32 nBufferlen;
            UInt32 nBufferUsed;

            UInt32 lBuffer;

            if (!bConnect)
                return 0;

            try
            {
                lBuffer = 0;
                nBufferlen = 1;
                //Debug.Print("SQLGetInfoShort for: " + CODBCReferences.sInformation[CODBCReferences.nInformation[vfInforType]] + ": " + vfInforType);
                SQLReturnCodes nSQLResults = Win32APISQL.SQLGetInfo(hdbc, (short)vfInforType,out lBuffer, nBufferlen, out nBufferUsed);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                {
                    //Debug.Print("SQLGetInfoShort value: " + lBuffer.ToString());
                    return (short)lBuffer;  // The cast is because the compiler is stupid.
                }
                else
                {
                    CheckError("SQLGetInfoShort: ", nSQLResults);
                    return 0;
                }
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                return 0;
            }
        }
        public String GetInfoString(int vfInforType)
        {
            //  Wrapper for the SQLGetInfo API

            UInt32 nBufferlen;
            UInt32 nBufferUsed;

            StringBuilder sbBuffer = new StringBuilder(1024);

            if (!bConnect)
                return "";

            try
            {
                nBufferlen = 1023;
                //Debug.Print("SQLGetInfoString for: " + CODBCReferences.sInformation[CODBCReferences.nInformation[vfInforType]] + ": " + vfInforType);
                SQLReturnCodes nSQLResults = Win32APISQL.SQLGetInfo(hdbc, (short)vfInforType, sbBuffer, nBufferlen, out nBufferUsed);
                if (SQLReturnCodes.SQL_SUCCESS == nSQLResults)
                {
                    //Debug.Print("SQLGetString value: " + sbBuffer.ToString());
                    return sbBuffer.ToString(0, (int)nBufferUsed);
                }
                else
                {
                    CheckError("SQLGetInfoString: ", nSQLResults);
                    return "";
                }
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                // Tell the user what happened
                MessageBoxResult result = MessageBox.Show(err.ToString(), consTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                return "";
            }
        }
        public void Information()
        {
            //  Get information items
            for (int nIndex = 1; nIndex <= connINFORMATION_USED; nIndex++)
            {
                if (null != CODBCReferences.sInformation[CODBCReferences.nInformation[nIndex]])
                {
                    switch (nIndex)
                    {
                        //
                        //   Strings
                        //
                        case (int)ODBCconst.SQL_DATA_SOURCE_NAME:
                        case (int)ODBCconst.SQL_DRIVER_NAME:
                        case (int)ODBCconst.SQL_DRIVER_VER:
                        case (int)ODBCconst.SQL_ODBC_VER:
                        case (int)ODBCconst.SQL_ROW_UPDATES:
                        case (int)ODBCconst.SQL_SERVER_NAME:
                        case (int)ODBCconst.SQL_SEARCH_PATTERN_ESCAPE:
                        case (int)ODBCconst.SQL_DBMS_NAME:
                        case (int)ODBCconst.SQL_DBMS_VER:
                        case (int)ODBCconst.SQL_ACCESSIBLE_TABLES:
                        case (int)ODBCconst.SQL_ACCESSIBLE_PROCEDURES:
                        case (int)ODBCconst.SQL_PROCEDURES:
                        case (int)ODBCconst.SQL_DATA_SOURCE_READ_ONLY:
                        case (int)ODBCconst.SQL_EXPRESSIONS_IN_ORDERBY:
                        case (int)ODBCconst.SQL_IDENTIFIER_QUOTE_CHAR:
                        case (int)ODBCconst.SQL_MULT_RESULT_SETS:
                        case (int)ODBCconst.SQL_MULTIPLE_ACTIVE_TXN:
                        case (int)ODBCconst.SQL_OUTER_JOINS:

                        case (int)ODBCconst.SQL_OWNER_TERM:
                        case (int)ODBCconst.SQL_PROCEDURE_TERM:
                        case (int)ODBCconst.SQL_QUALIFIER_NAME_SEPARATOR:
                        case (int)ODBCconst.SQL_QUALIFIER_TERM:
                        case (int)ODBCconst.SQL_TABLE_TERM:
                        case (int)ODBCconst.SQL_USER_NAME:
                        case (int)ODBCconst.SQL_ODBC_SQL_OPT_IEF:
                        case (int)ODBCconst.SQL_DRIVER_ODBC_VER:
                        case (int)ODBCconst.SQL_COLUMN_ALIAS:

                        case (int)ODBCconst.SQL_KEYWORDS:
                        case (int)ODBCconst.SQL_ORDER_BY_COLUMNS_IN_SELECT:
                        case (int)ODBCconst.SQL_SPECIAL_CHARACTERS:
                        case (int)ODBCconst.SQL_MAX_ROW_SIZE_INCLUDES_LONG:
                        case (int)ODBCconst.SQL_NEED_LONG_DATA_LEN:
                        case (int)ODBCconst.SQL_LIKE_ESCAPE_CLAUSE:
                            sInformationString[CODBCReferences.nInformation[nIndex]] = GetInfoString(nIndex);
                            break;

                        //
                        //   Long Values - Decimal
                        //
                        case (int)ODBCconst.SQL_MAX_BINARY_LITERAL_LEN:
                        case (int)ODBCconst.SQL_MAX_CHAR_LITERAL_LEN:
                        case (int)ODBCconst.SQL_MAX_INDEX_SIZE:
                        case (int)ODBCconst.SQL_MAX_ROW_SIZE:
                        case (int)ODBCconst.SQL_MAX_STATEMENT_LEN:
                            lInformationLong[CODBCReferences.nInformation[nIndex]] = GetInfoLong(nIndex);
                            sInformationString[CODBCReferences.nInformation[nIndex]] = lInformationLong[CODBCReferences.nInformation[nIndex]].ToString("#,##0");
                            break;

                        //
                        //   Short Values - Decimal
                        //
                        case (int)ODBCconst.SQL_ACTIVE_CONNECTIONS:
                        case (int)ODBCconst.SQL_ACTIVE_STATEMENTS:
                        case (int)ODBCconst.SQL_MAX_COLUMNS_IN_GROUP_BY:
                        case (int)ODBCconst.SQL_MAX_COLUMNS_IN_INDEX:
                        case (int)ODBCconst.SQL_MAX_COLUMNS_IN_ORDER_BY:
                        case (int)ODBCconst.SQL_MAX_COLUMNS_IN_SELECT:
                        case (int)ODBCconst.SQL_MAX_COLUMNS_IN_TABLE:
                        case (int)ODBCconst.SQL_MAX_COLUMN_NAME_LEN:
                        case (int)ODBCconst.SQL_MAX_CURSOR_NAME_LEN:
                        case (int)ODBCconst.SQL_MAX_OWNER_NAME_LEN:

                        case (int)ODBCconst.SQL_MAX_PROCEDURE_NAME_LEN:
                        case (int)ODBCconst.SQL_MAX_QUALIFIER_NAME_LEN:
                        case (int)ODBCconst.SQL_MAX_TABLES_IN_SELECT:
                        case (int)ODBCconst.SQL_MAX_TABLE_NAME_LEN:
                        case (int)ODBCconst.SQL_MAX_USER_NAME_LEN:
                            nInformationShort[CODBCReferences.nInformation[nIndex]] = GetInfoShort(nIndex);
                            sInformationString[CODBCReferences.nInformation[nIndex]] = nInformationShort[CODBCReferences.nInformation[nIndex]].ToString("#,##0");
                            break;

                        //
                        //   Long Values - Hex
                        //
                        case (int)ODBCconst.SQL_DRIVER_HDBC:
                        case (int)ODBCconst.SQL_DRIVER_HENV:
                        case (int)ODBCconst.SQL_DRIVER_HLIB:
                        case (int)ODBCconst.SQL_DRIVER_HSTMT:
                            lInformationLong[CODBCReferences.nInformation[nIndex]] = GetInfoLong(nIndex);
                            sInformationString[CODBCReferences.nInformation[nIndex]] = ("00000000" + lInformationLong[CODBCReferences.nInformation[nIndex]].ToString("X8")).Substring(lInformationLong[CODBCReferences.nInformation[nIndex]].ToString("X8").Length, 8);
                            break;

                        //
                        //   Long Flags - Conversions
                        //
                        case (int)ODBCconst.SQL_CONVERT_BIGINT:
                        case (int)ODBCconst.SQL_CONVERT_BINARY:
                        case (int)ODBCconst.SQL_CONVERT_BIT:
                        case (int)ODBCconst.SQL_CONVERT_CHAR:
                        case (int)ODBCconst.SQL_CONVERT_DATE:

                        case (int)ODBCconst.SQL_CONVERT_DECIMAL:
                        case (int)ODBCconst.SQL_CONVERT_DOUBLE:
                        case (int)ODBCconst.SQL_CONVERT_FLOAT:
                        case (int)ODBCconst.SQL_CONVERT_INTEGER:
                        case (int)ODBCconst.SQL_CONVERT_LONGVARCHAR:
                        case (int)ODBCconst.SQL_CONVERT_NUMERIC:
                        case (int)ODBCconst.SQL_CONVERT_REAL:
                        case (int)ODBCconst.SQL_CONVERT_SMALLINT:
                        case (int)ODBCconst.SQL_CONVERT_TIME:
                        case (int)ODBCconst.SQL_CONVERT_TIMESTAMP:

                        case (int)ODBCconst.SQL_CONVERT_TINYINT:
                        case (int)ODBCconst.SQL_CONVERT_VARBINARY:
                        case (int)ODBCconst.SQL_CONVERT_VARCHAR:
                        case (int)ODBCconst.SQL_CONVERT_LONGVARBINARY:
                            lInformationLong[CODBCReferences.nInformation[nIndex]] = GetInfoLong(nIndex);
                            string sMessage = "";
                            if (0 != ((int)ODBCconst.SQL_CVT_CHAR | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "CHAR" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_NUMERIC | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "NUMERIC" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_DECIMAL | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "DECIMAL" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_INTEGER | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "INTEGER" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_SMALLINT | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "SMALLINT" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_FLOAT | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "FLOAT" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_REAL | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "REAL" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_DOUBLE | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "DOUBLE" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_VARCHAR | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "VARCHAR" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_LONGVARCHAR | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "LONGVARCHAR" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_BINARY | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "BINARY" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_VARBINARY | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "VARBINARY" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_BIT | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "BIT" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_TINYINT | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "TINYINT" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_BIGINT | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "BIGINT" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_DATE | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "DATE" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_TIME | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "TIME" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_TIMESTAMP | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "TIMESTAMP" + " ";
                            if (0 != ((int)ODBCconst.SQL_CVT_LONGVARBINARY | lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "LONGVARBINARY" + " ";
                            sInformationString[CODBCReferences.nInformation[nIndex]] = sMessage;
                            break;

                        //
                        //   Long Flags - Other than Conversions
                        //
                        case (int)ODBCconst.SQL_ALTER_TABLE:
                        case (int)ODBCconst.SQL_BOOKMARK_PERSISTENCE:
                        case (int)ODBCconst.SQL_CONVERT_FUNCTIONS:
                        case (int)ODBCconst.SQL_DEFAULT_TXN_ISOLATION:
                        case (int)ODBCconst.SQL_FETCH_DIRECTION:

                        case (int)ODBCconst.SQL_GETDATA_EXTENSIONS:
                        case (int)ODBCconst.SQL_LOCK_TYPES:
                        case (int)ODBCconst.SQL_NUMERIC_FUNCTIONS:
                        case (int)ODBCconst.SQL_OWNER_USAGE:
                        case (int)ODBCconst.SQL_POSITIONED_STATEMENTS:

                        case (int)ODBCconst.SQL_POS_OPERATIONS:
                        case (int)ODBCconst.SQL_QUALIFIER_USAGE:
                        case (int)ODBCconst.SQL_QUOTED_IDENTIFIER_CASE:
                        case (int)ODBCconst.SQL_SCROLL_CONCURRENCY:
                        case (int)ODBCconst.SQL_SCROLL_OPTIONS:

                        case (int)ODBCconst.SQL_STATIC_SENSITIVITY:
                        case (int)ODBCconst.SQL_STRING_FUNCTIONS:
                        case (int)ODBCconst.SQL_SUBQUERIES:
                        case (int)ODBCconst.SQL_SYSTEM_FUNCTIONS:
                        case (int)ODBCconst.SQL_TIMEDATE_ADD_INTERVALS:

                        case (int)ODBCconst.SQL_TIMEDATE_DIFF_INTERVALS:
                        case (int)ODBCconst.SQL_TIMEDATE_FUNCTIONS:
                        case (int)ODBCconst.SQL_TXN_CAPABLE:
                        case (int)ODBCconst.SQL_TXN_ISOLATION_OPTION:
                        case (int)ODBCconst.SQL_UNION:
                            lInformationLong[CODBCReferences.nInformation[nIndex]] = GetInfoLong(nIndex);
                            sMessage = "";
                            switch (nIndex)
                            {
                                case (int)ODBCconst.SQL_ALTER_TABLE:
                                    if (0 != ((int)ODBCconst.SQL_AT_ADD_COLUMN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Add Column ";
                                    if (0 != ((int)ODBCconst.SQL_AT_DROP_COLUMN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Drop Column ";
                                    break;
                                case (int)ODBCconst.SQL_BOOKMARK_PERSISTENCE:
                                    if (0 != ((int)ODBCconst.SQL_BP_CLOSE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Close ";
                                    if (0 != ((int)ODBCconst.SQL_BP_DELETE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Delete ";
                                    if (0 != ((int)ODBCconst.SQL_BP_DROP & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Drop ";
                                    if (0 != ((int)ODBCconst.SQL_BP_TRANSACTION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Transaction ";
                                    if (0 != ((int)ODBCconst.SQL_BP_UPDATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Update ";
                                    if (0 != ((int)ODBCconst.SQL_BP_OTHER_HSTMT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Other hstmt ";
                                    if (0 != ((int)ODBCconst.SQL_BP_SCROLL & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Scroll ";
                                    break;
                                case (int)ODBCconst.SQL_CONVERT_FUNCTIONS:
                                    if (0 != ((int)ODBCconst.SQL_FN_CVT_CONVERT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Convert ";
                                    break;
                                case (int)ODBCconst.SQL_DEFAULT_TXN_ISOLATION:
                                    if (0 != ((int)ODBCconst.SQL_TXN_READ_UNCOMMITTED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "READ_UNCOMMITTED: Dirty Reads, Nonrepeatable Reads & Phatoms are possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_READ_COMMITTED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "READ_COMMITTED: Dirty Reads are not possible, Nonrepeatable Reads & Phatoms are possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_REPEATABLE_READ & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "REPEATABLE_READ: Dirty Reads & Nonrepeatable Reads are not possible, Phatoms are possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_SERIALIZABLE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "SERIALIZABLE: Dirty Reads, Nonrepeatable Reads & Phatoms are not possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_VERSIONING & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "VERSIONING: Dirty Reads, Nonrepeatable Reads & Phatoms are not possible ";
                                    break;
                                case (int)ODBCconst.SQL_FETCH_DIRECTION:
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_NEXT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Next ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_FIRST & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "First ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_LAST & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Last ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_PRIOR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Prior ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_ABSOLUTE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Absolute ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_RELATIVE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Relative ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_RESUME & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Resume ";
                                    if (0 != ((int)ODBCconst.SQL_FD_FETCH_BOOKMARK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Bookmark ";
                                    break;
                                case (int)ODBCconst.SQL_GETDATA_EXTENSIONS:
                                    if (0 != ((int)ODBCconst.SQL_GD_ANY_COLUMN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Any Column ";
                                    if (0 != ((int)ODBCconst.SQL_GD_ANY_ORDER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Any Order ";
                                    if (0 != ((int)ODBCconst.SQL_GD_BLOCK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Block ";
                                    if (0 != ((int)ODBCconst.SQL_GD_BOUND & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Bound ";
                                    break;
                                case (int)ODBCconst.SQL_LOCK_TYPES:
                                    if (0 != ((int)ODBCconst.SQL_LCK_NO_CHANGE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "No Change ";
                                    if (0 != ((int)ODBCconst.SQL_LCK_EXCLUSIVE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Exclusive ";
                                    if (0 != ((int)ODBCconst.SQL_LCK_UNLOCK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Unlock ";
                                    break;
                                case (int)ODBCconst.SQL_NUMERIC_FUNCTIONS:
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_ACOS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "ACos ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_ASIN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "ASin ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_ATAN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "ATan ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_ATAN2 & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "ATan2 ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_CEILING & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Ceiling ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_COS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Cos ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_COT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "COT ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_EXP & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Exp ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_FLOOR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Floor ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_LOG & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Log ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_MOD & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Mod ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_SIGN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Sign ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_SIN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Sin ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_SQRT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Sqrt ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_TAN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Tan ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_PI & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Pi ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_RAND & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Rand ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_DEGREES & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Degrees ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_LOG10 & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Log10 ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_POWER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Power ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_RADIANS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Radians ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_ROUND & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Round ";
                                    if (0 != ((int)ODBCconst.SQL_FN_NUM_TRUNCATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Truncate ";
                                    break;
                                case (int)ODBCconst.SQL_OWNER_USAGE:
                                    if (0 != ((int)ODBCconst.SQL_OU_DML_STATEMENTS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "All DML ";
                                    if (0 != ((int)ODBCconst.SQL_OU_PROCEDURE_INVOCATION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Procedure Invocation ";
                                    if (0 != ((int)ODBCconst.SQL_OU_TABLE_DEFINITION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Table Definitions  ";
                                    if (0 != ((int)ODBCconst.SQL_OU_INDEX_DEFINITION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Index Definitions ";
                                    if (0 != ((int)ODBCconst.SQL_OU_PRIVILEGE_DEFINITION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Privilege Definition ";
                                    break;
                                case (int)ODBCconst.SQL_POSITIONED_STATEMENTS:
                                    if (0 != ((int)ODBCconst.SQL_PS_POSITIONED_DELETE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Delete ";
                                    if (0 != ((int)ODBCconst.SQL_PS_POSITIONED_UPDATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Update ";
                                    if (0 != ((int)ODBCconst.SQL_PS_SELECT_FOR_UPDATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "switch for Update ";
                                    break;
                                case (int)ODBCconst.SQL_POS_OPERATIONS:
                                    if (0 != ((int)ODBCconst.SQL_POS_POSITION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Position ";
                                    if (0 != ((int)ODBCconst.SQL_POS_REFRESH & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Refresh ";
                                    if (0 != ((int)ODBCconst.SQL_POS_UPDATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Update ";
                                    if (0 != ((int)ODBCconst.SQL_POS_DELETE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Delete ";
                                    if (0 != ((int)ODBCconst.SQL_POS_ADD & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Add ";
                                    break;
                                case (int)ODBCconst.SQL_QUALIFIER_USAGE:
                                    if (0 != ((int)ODBCconst.SQL_QL_START & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Start ";
                                    if (0 != ((int)ODBCconst.SQL_QL_END & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "End ";
                                    break;
                                case (int)ODBCconst.SQL_QUOTED_IDENTIFIER_CASE:
                                    if (0 != ((int)ODBCconst.SQL_IC_UPPER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case insensitive, Stored upper ";
                                    if (0 != ((int)ODBCconst.SQL_IC_LOWER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case insensitive, Stored lower ";
                                    if (0 != ((int)ODBCconst.SQL_IC_SENSITIVE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case sensitive, Stored mixed ";
                                    if (0 != ((int)ODBCconst.SQL_IC_MIXED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case insensitive, Stored mixed ";
                                    break;
                                case (int)ODBCconst.SQL_SCROLL_CONCURRENCY:
                                    if (0 != ((int)ODBCconst.SQL_SCCO_READ_ONLY & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Read only, No updates ";
                                    if (0 != ((int)ODBCconst.SQL_SCCO_LOCK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Lowest level of locking, updates allowed ";
                                    if (0 != ((int)ODBCconst.SQL_SCCO_OPT_ROWVER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Optimistic concurrency control, comparing row versions ";
                                    if (0 != ((int)ODBCconst.SQL_SCCO_OPT_VALUES & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Optimistic concurrency control, comparing values ";
                                    break;
                                case (int)ODBCconst.SQL_SCROLL_OPTIONS:
                                    if (0 != ((int)ODBCconst.SQL_SO_FORWARD_ONLY & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Cursor only scrolls forward ";
                                    if (0 != ((int)ODBCconst.SQL_SO_KEYSET_DRIVEN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Saves & uses keys for every row ";
                                    if (0 != ((int)ODBCconst.SQL_SO_DYNAMIC & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Keeps & uses keys for every row ";
                                    if (0 != ((int)ODBCconst.SQL_SO_MIXED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Keyset-driven inside the keyset & dynamic outside ";
                                    break;
                                case (int)ODBCconst.SQL_STATIC_SENSITIVITY:
                                    if (0 != ((int)ODBCconst.SQL_SS_ADDITIONS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Added rows are visible to the cursor ";
                                    if (0 != ((int)ODBCconst.SQL_SS_DELETIONS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Deleted rows are no longer visible to the cursor ";
                                    if (0 != ((int)ODBCconst.SQL_SS_UPDATES & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Updated rows are visible to the cursor ";
                                    break;
                                case (int)ODBCconst.SQL_STRING_FUNCTIONS:
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_CONCAT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Concat ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_INSERT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Insert ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_LEFT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Left ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_LTRIM & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Ltrim ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_LENGTH & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Length ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_LOCATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Locate ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_LCASE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Lcase ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_REPEAT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Repeat ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_REPLACE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Replace ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_RIGHT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Right ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_RTRIM & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Rtrim ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_SUBSTRING & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Substring ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_UCASE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Ucase ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_ASCII & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Ascii ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_CHAR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Char ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_DIFFERENCE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Difference ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_LOCATE_2 & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Locate_2 ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_SOUNDEX & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Soundex ";
                                    if (0 != ((int)ODBCconst.SQL_FN_STR_SPACE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Space ";
                                    break;
                                case (int)ODBCconst.SQL_SUBQUERIES:
                                    if (0 != ((int)ODBCconst.SQL_SQ_COMPARISON & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Comparison ";
                                    if (0 != ((int)ODBCconst.SQL_SQ_EXISTS & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Exists ";
                                    if (0 != ((int)ODBCconst.SQL_SQ_IN & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "In ";
                                    if (0 != ((int)ODBCconst.SQL_SQ_QUANTIFIED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Quantified ";
                                    if (0 != ((int)ODBCconst.SQL_SQ_CORRELATED_SUBQUERIES & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Correlated ";
                                    break;
                                case (int)ODBCconst.SQL_SYSTEM_FUNCTIONS:
                                    if (0 != ((int)ODBCconst.SQL_FN_SYS_USERNAME & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Username ";
                                    if (0 != ((int)ODBCconst.SQL_FN_SYS_DBNAME & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "DBName ";
                                    if (0 != ((int)ODBCconst.SQL_FN_SYS_IFNULL & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "IfNull ";
                                    break;
                                case (int)ODBCconst.SQL_TIMEDATE_ADD_INTERVALS:
                                case (int)ODBCconst.SQL_TIMEDATE_DIFF_INTERVALS:
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_FRAC_SECOND & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Frac Second ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_SECOND & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Second ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_MINUTE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Minute ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_HOUR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Hour ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_DAY & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Day ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_WEEK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Week ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_MONTH & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Month ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_QUARTER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Quarter ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TSI_YEAR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Year ";
                                    break;
                                case (int)ODBCconst.SQL_TIMEDATE_FUNCTIONS:
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_NOW & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Now ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_CURDATE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Curdate ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_DAYOFMONTH & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Dayofmonth ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_DAYOFWEEK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Dayofweek ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_DAYOFYEAR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Dayofyear ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_MONTH & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Month ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_QUARTER & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Quarter ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_WEEK & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Week ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_YEAR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Year ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_CURTIME & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Curtime ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_HOUR & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Hour ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_MINUTE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Minute ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_SECOND & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Second ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_TIMESTAMPADD & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Timestampadd ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_TIMESTAMPDIFF & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Timestampdiff ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_DAYNAME & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Dayname ";
                                    if (0 != ((int)ODBCconst.SQL_FN_TD_MONTHNAME & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Monthname ";
                                    break;
                                case (int)ODBCconst.SQL_TXN_CAPABLE:
                                    if (0 != ((int)ODBCconst.SQL_TC_NONE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Transactions not supported ";
                                    if (0 != ((int)ODBCconst.SQL_TC_DML & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Transactions only DML (switch, Insert, Update, Delete) ";
                                    if (0 != ((int)ODBCconst.SQL_TC_ALL & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Transactions with DDL & DML ";
                                    if (0 != ((int)ODBCconst.SQL_TC_DDL_COMMIT & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Transactions only DML - DDL forces commit ";
                                    if (0 != ((int)ODBCconst.SQL_TC_DDL_IGNORE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Transactions only DML - DDL ignored ";
                                    break;
                                case (int)ODBCconst.SQL_TXN_ISOLATION_OPTION:
                                    if (0 != ((int)ODBCconst.SQL_TXN_READ_UNCOMMITTED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "READ_UNCOMMITTED: Dirty Reads, Nonrepeatable Reads & Phatoms are possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_READ_COMMITTED & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "READ_COMMITTED: Dirty Reads are not possible, Nonrepeatable Reads & Phatoms are possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_REPEATABLE_READ & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "REPEATABLE_READ: Dirty Reads & Nonrepeatable Reads are not possible, Phatoms are possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_SERIALIZABLE & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "SERIALIZABLE: Dirty Reads, Nonrepeatable Reads & Phatoms are not possible ";
                                    if (0 != ((int)ODBCconst.SQL_TXN_VERSIONING & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "VERSIONING: Dirty Reads, Nonrepeatable Reads & Phatoms are not possible ";
                                    break;
                                case (int)ODBCconst.SQL_UNION:
                                    if (0 != ((int)ODBCconst.SQL_U_UNION & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Union supported ";
                                    if (0 != ((int)ODBCconst.SQL_U_UNION_ALL & lInformationLong[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "with ALL keyword ";
                                    break;
                                default:
                                    break;
                            }
                            sInformationString[CODBCReferences.nInformation[nIndex]] = sMessage;
                            break;

                        //
                        //   Short Flags
                        //
                        case (int)ODBCconst.SQL_CONCAT_NULL_BEHAVIOR:
                        case (int)ODBCconst.SQL_CORRELATION_NAME:
                        case (int)ODBCconst.SQL_CURSOR_COMMIT_BEHAVIOR:
                        case (int)ODBCconst.SQL_CURSOR_ROLLBACK_BEHAVIOR:
                        case (int)ODBCconst.SQL_FILE_USAGE:

                        case (int)ODBCconst.SQL_GROUP_BY:
                        case (int)ODBCconst.SQL_IDENTIFIER_CASE:
                        case (int)ODBCconst.SQL_NON_NULLABLE_COLUMNS:
                        case (int)ODBCconst.SQL_NULL_COLLATION:
                        case (int)ODBCconst.SQL_ODBC_API_CONFORMANCE:

                        case (int)ODBCconst.SQL_ODBC_SAG_CLI_CONFORMANCE:
                        case (int)ODBCconst.SQL_ODBC_SQL_CONFORMANCE:
                        case (int)ODBCconst.SQL_QUALIFIER_LOCATION:
                            nInformationShort[CODBCReferences.nInformation[nIndex]] = GetInfoShort(nIndex);
                            sMessage = "";

                            switch (nIndex)
                            {
                                case (int)ODBCconst.SQL_CONCAT_NULL_BEHAVIOR:
                                    if (0 != ((int)ODBCconst.SQL_CB_NULL & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "NULL ";
                                    if (0 != ((int)ODBCconst.SQL_CB_NON_NULL & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Concatenation of non-NULL value of column or columns ";
                                    break;
                                case (int)ODBCconst.SQL_CORRELATION_NAME:
                                    if (0 != ((int)ODBCconst.SQL_CN_NONE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Correlation names are not supported ";
                                    if (0 != ((int)ODBCconst.SQL_CN_DIFFERENT & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Correlation names are supported but must differ from tables ";
                                    if (0 != ((int)ODBCconst.SQL_CN_ANY & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Correlation names are supported ";
                                    break;
                                case (int)ODBCconst.SQL_CURSOR_COMMIT_BEHAVIOR:
                                    if (0 != ((int)ODBCconst.SQL_CB_DELETE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Close cursors & delete prepared statements ";
                                    if (0 != ((int)ODBCconst.SQL_CB_CLOSE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Close cursors ";
                                    if (0 != ((int)ODBCconst.SQL_CB_PRESERVE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Cursors preserved ";
                                    break;
                                case (int)ODBCconst.SQL_CURSOR_ROLLBACK_BEHAVIOR:
                                    if (0 != ((int)ODBCconst.SQL_CB_DELETE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Close cursors & delete prepared statements ";
                                    if (0 != ((int)ODBCconst.SQL_CB_CLOSE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Close cursors ";
                                    if (0 != ((int)ODBCconst.SQL_CB_PRESERVE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Cursors preserved ";
                                    break;
                                case (int)ODBCconst.SQL_FILE_USAGE:
                                    if (0 != ((int)ODBCconst.SQL_FILE_NOT_SUPPORTED & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "The driver is not a single tier driver ";
                                    if (0 != ((int)ODBCconst.SQL_FILE_TABLE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Treated as table ";
                                    if (0 != ((int)ODBCconst.SQL_FILE_QUALIFIER & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Treated as qualifier ";
                                    break;
                                case (int)ODBCconst.SQL_GROUP_BY:
                                    if (0 != ((int)ODBCconst.SQL_GB_NOT_SUPPORTED & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Group By clauses not supported ";
                                    if (0 != ((int)ODBCconst.SQL_GB_GROUP_BY_EQUALS_SELECT & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Must contain all & only non-aggregated columns in switch ";
                                    if (0 != ((int)ODBCconst.SQL_GB_GROUP_BY_CONTAINS_SELECT & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Must contain all non-aggregated columns in switch & may contain others ";
                                    if (0 != ((int)ODBCconst.SQL_GB_NO_RELATION & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Can contain any columns ";
                                    break;
                                case (int)ODBCconst.SQL_IDENTIFIER_CASE:
                                    if (0 != ((int)ODBCconst.SQL_IC_UPPER & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case insensitive, Stored upper ";
                                    if (0 != ((int)ODBCconst.SQL_IC_LOWER & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case insensitive, Stored lower ";
                                    if (0 != ((int)ODBCconst.SQL_IC_SENSITIVE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case sensitive, Stored mixed ";
                                    if (0 != ((int)ODBCconst.SQL_IC_MIXED & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Case insensitive, Stored mixed ";
                                    break;
                                case (int)ODBCconst.SQL_NON_NULLABLE_COLUMNS:
                                    if (0 != ((int)ODBCconst.SQL_NNC_NULL & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "All columns must be nullable ";
                                    if (0 != ((int)ODBCconst.SQL_NNC_NON_NULL & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Columns may be non-nullable ";
                                    break;
                                case (int)ODBCconst.SQL_NULL_COLLATION:
                                    if (0 != ((int)ODBCconst.SQL_NC_HIGH & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Nulls sorted high ";
                                    if (0 != ((int)ODBCconst.SQL_NC_LOW & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Nulls sorted low ";
                                    if (0 != ((int)ODBCconst.SQL_NC_START & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Nulls always at start ";
                                    if (0 != ((int)ODBCconst.SQL_NC_END & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Nulls always at end ";
                                    break;
                                case (int)ODBCconst.SQL_ODBC_API_CONFORMANCE:
                                    if (0 != ((int)ODBCconst.SQL_OAC_NONE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "None ";
                                    if (0 != ((int)ODBCconst.SQL_OAC_LEVEL1 & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Level 1 ";
                                    if (0 != ((int)ODBCconst.SQL_OAC_LEVEL2 & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Level 2 ";
                                    break;
                                case (int)ODBCconst.SQL_ODBC_SAG_CLI_CONFORMANCE:
                                    if (0 != ((int)ODBCconst.SQL_OSCC_NOT_COMPLIANT & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Not SAG-compliant ";
                                    if (0 != ((int)ODBCconst.SQL_OSCC_COMPLIANT & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "SAG-compliant ";
                                    break;
                                case (int)ODBCconst.SQL_ODBC_SQL_CONFORMANCE:
                                    if (0 != ((int)ODBCconst.SQL_OSC_MINIMUM & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Minimum grammar supported ";
                                    if (0 != ((int)ODBCconst.SQL_OSC_CORE & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Core grammar supported ";
                                    if (0 != ((int)ODBCconst.SQL_OSC_EXTENDED & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Extended grammar supported ";
                                    break;
                                case (int)ODBCconst.SQL_QUALIFIER_LOCATION:
                                    if (0 != ((int)ODBCconst.SQL_QL_START & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Qualifier at start ";
                                    if (0 != ((int)ODBCconst.SQL_QL_END & nInformationShort[CODBCReferences.nInformation[nIndex]])) sMessage = sMessage + "Qualifier at end ";
                                    break;
                                default:
                                    break;
                            }
                            sInformationString[CODBCReferences.nInformation[nIndex]] = sMessage;
                            break;
                    }
                    //Debug.Print(CODBCReferences.sInformation[CODBCReferences.nInformation[nIndex]] + "|" + sInformationString[CODBCReferences.nInformation[nIndex]]);
                    {
                    }
                }
            }
        }
        public string InformationName(int nIndex)
        {
            return CODBCReferences.sInformation[nIndex];
        }
        public string InformationString(int nIndex)
        {
            return sInformationString[nIndex];
        }
        public int InformationUsed()
        {
            return connINFORMATION_USED;
        }
    }
}

