﻿/*
 * Written by Stewart Berman 2012
 * Please send any bug reports / suggestions to sab@saberman.com
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Globalization;
using Microsoft.Win32;

namespace ODBC
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class winDataSourceNames : Window
    {
        clsODBCReferences CODBCReferences = clsODBCReferences.Instance;

        Cursor OldCursor;

        int nDataSources;
        int nDataSourcesLenMax;

        String sActiveDataSource = "";

        Hashtable ODBCSources = new Hashtable();

        public winDataSourceNames()
        {
            InitializeComponent();
            InitializeDataSources();
            //  Expose the window
            this.Show();
            //  Create our companion window, position it and show it
            var winManualDataSources = new winManualDataSources();
            winManualDataSources.Top = this.Top;
            winManualDataSources.Left = this.Left + this.Width + 50;
            winManualDataSources.Show();
            //  Create our splash screen, position it and show it
            var winAbout = new winAbout();
            winAbout.Top = this.Top + this.Height/4;
            winAbout.Left = this.Left + this.Width/2 + 25;
            winAbout.ShowDialog();
        }
        private void btnExit_Click(object sender, RoutedEventArgs e)
        {
            //  Exit application
            Application.Current.Shutdown();
        }
        private void btnGetInformation_Click(object sender, RoutedEventArgs e)
        {
            clsODBC CODBC;

            //  Did the user seleect an openned DSN to use?
            if ((0 >= lbDataSourcesOpen.Items.Count) || (0 > lbDataSourcesOpen.SelectedIndex))
            {
                //  No -- tell the user and get out
                MessageBox.Show("Please select an open data source and try again.", "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            //  Disable all of the buttons on the form and change the cursor to busy
            DisableAllButtonsOnForm();

            try
            {
                //  Get the data source associated with the selected entry
                CODBC = (clsODBC)ODBCSources[lbDataSourcesOpen.SelectedItem.ToString().Trim()];
                //  Find out which functions are supported
                CODBC.Functions();
                //  Update the function listbox
                int nFunction = 0;
                for (int nIndex = 1; nIndex <= CODBC.FunctionsUsed(); nIndex++)
                {
                    //  For each function that we setup update the entry in the listbox
                    if (CODBC.FunctionExists(nIndex))
                    {
                        nFunction++;
                        if (CODBC.Function(nIndex))
                            lbFunctions.Items[nFunction] = lbFunctions.Items[nFunction].ToString().Substring(0, 31) + "Yes";
                        else
                            lbFunctions.Items[nFunction] = lbFunctions.Items[nFunction].ToString().Substring(0, 31) + "No";
                        //Debug.Print(nFunction.ToString("000") + "|" + lbFunctions.Items[nFunction].ToString());
                    }
                }
                //  Get information values
                CODBC.Information();
                // Get information items
                int nInformation = 0;
                for (int nIndex = 1; nIndex < CODBC.InformationUsed(); nIndex++)
                {
                    if (!(CODBC.InformationName(nIndex) == null))
                    {
                        nInformation++;
                        if (0 != CODBCReferences.nInformation[nInformation])
                        {
                            lbInformation.Items[(int)CODBCReferences.nInformation[nInformation]] = lbInformation.Items[(int)CODBCReferences.nInformation[nInformation]].ToString().Substring(0, 31) + CODBC.InformationString((int)CODBCReferences.nInformation[nInformation]);
                            Debug.Print(nInformation.ToString() + "|" + lbInformation.Items[(int)CODBCReferences.nInformation[nInformation]].ToString());
                        }
                    }
                }
                sActiveDataSource = lbDataSourcesOpen.SelectedItem.ToString().Trim();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
            finally
            {
                lbDataSourcesOpen.UnselectAll();
                CODBC = null;
                //  Enable all of the buttons on the form and restore the cursor
                EnableAllButtonsOnForm();
            }
        }
        private void btnOpenConnection_Click(object sender, RoutedEventArgs e)
        {
            clsODBC CODBC;

            //  Do we have a data source selected?
            if ((0 >= lbDataSourceNames.Items.Count) || (0 >= lbDataSourceNames.SelectedIndex))
            {
                //  No -- tell the user and get out
                MessageBox.Show("Please select a data source and try again.", "Information for ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            //  Disable all of the buttons and change the cursor to busy
            DisableAllButtonsOnForm();

            try
            {
                //  Allocate an env and connection
                CODBC = new clsODBC(CODBCReferences);
                String sDataSource = lbDataSourceNames.SelectedItem.ToString().Substring(0, nDataSourcesLenMax).Trim();
                CODBC.AllocEnv();
                CODBC.AllocConnect();
                //  Connect to the database
                if (CODBC.Connect(sDataSource, txtUserid.Text.ToString(), txtPassword.Password.ToString()))
                {
                    //  Connection successfull -- allocate a statement and add the data source to the collection
                    lbDataSourcesOpen.Items.Add(sDataSource);
                    CODBC.AllocStmt();
                    ODBCSources.Add(sDataSource, CODBC);
                }

            }
            catch (Exception ex)
            {
                //  Caught an error -- tell the user
                MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
            finally
            {
                CODBC = null;
                EnableAllButtonsOnForm();
            }
        }
        private void btnSaveInformation_Click(object sender, RoutedEventArgs e)
        {
            if (0 == sActiveDataSource.Length)
            {
                MessageBox.Show("Please retrieve information before trying to save it.", "Information for ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            // Configure open file dialog box
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.CheckPathExists = true;
            ofd.CheckFileExists = false;
            ofd.DefaultExt = ".txt";
            ofd.FileName = sActiveDataSource.Replace("|", "_") + "_" + DateTime.Now.ToString("yyyyMMddHHmmss");
            ofd.Filter = "Text documents (.txt)|*.txt|All files (*.*)|*.*";
            ofd.FilterIndex = 1;
            ofd.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            ofd.Multiselect = false;
            ofd.Title = "Information for ODBC";

            // Show open file dialog box
            Nullable<bool> result = ofd.ShowDialog();

            // Process open file dialog box results
            if (result == true)
            {
                try
                {
                    // Get data source
                    clsODBC CODBC = (clsODBC)ODBCSources[sActiveDataSource];
                    // Open document
                    StreamWriter sw = new StreamWriter(ofd.FileName, false);
                    sw.WriteLine("Information About ODBC");
                    sw.WriteLine("Data Soure: " + sActiveDataSource);
                    sw.WriteLine("Connect String: " + CODBC.ConnectStringOut());
                    sw.WriteLine();
                    for (int nIndex = 0; nIndex < lbFunctions.Items.Count; nIndex++)
                        sw.WriteLine(lbFunctions.Items[nIndex].ToString());
                    sw.WriteLine();
                    for (int nIndex = 0; nIndex < lbInformation.Items.Count; nIndex++)
                        sw.WriteLine(lbInformation.Items[nIndex].ToString());
                    sw.Close();
                    sw = null;
                    CODBC = null;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                }
            }
        }
        private void DisableAllButtonsOnForm()
        {
            //  Disable the buttons while we do something so the user doesn't accidently click one
            this.btnExit.IsEnabled = false;
            this.btnGetInformation.IsEnabled = false;
            this.btnOpenConnection.IsEnabled = false;
            this.btnSaveInformation.IsEnabled = false;
            // Save the old mouse pointer and change it to busy
            OldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Wait;
        }
        private void EnableAllButtonsOnForm()
        {
            //  Enable the buttons after we finish what we were doing
            this.btnExit.IsEnabled = true;
            this.btnGetInformation.IsEnabled = true;
            this.btnOpenConnection.IsEnabled = true;
            this.btnSaveInformation.IsEnabled = true;
            //  Reset the cursor
            Mouse.OverrideCursor = OldCursor;
        }
        private void InitializeDataSources()
        {
            clsODBC CODBC;

            int nIndex;

            string[] sDataSources;
            string[] sDescriptons;

            // Create the clsODBC class and have it get the data sources
            CODBC = new clsODBC(CODBCReferences);
            nDataSources = CODBC.DataSources();
            nDataSourcesLenMax = 0;
            //  Do we have any data sources?
            if (0 < nDataSources)
            {
                //  Yes -- copy the names into our arrays
                sDataSources = new string[nDataSources];
                sDescriptons = new string[nDataSources];
                for (nIndex = 0; nIndex < nDataSources; nIndex++)
                {
                    sDataSources[nIndex] = CODBC.DSN(nIndex);
                    sDescriptons[nIndex] = CODBC.Description(nIndex);
                    //  We need the maximum length of the data source names for formatting
                    if (nDataSourcesLenMax < sDataSources[nIndex].Length)
                        nDataSourcesLenMax = sDataSources[nIndex].Length;
                }
                //  Need to padd out the data source names
                String sPadding = new String(' ', nDataSourcesLenMax + 1);
                //  Combine the data source names and their description and sort them
                SortedSet<String> sl = new SortedSet<String>();
                for (nIndex = 0; nIndex < nDataSources; nIndex++)
                    sl.Add(sDataSources[nIndex] + sPadding.Substring(0, 1 + nDataSourcesLenMax - sDataSources[nIndex].Length) + sDescriptons[nIndex]);
                //  Put headings in listbox
                lbDataSourceNames.Items.Add("Data Source Name" + sPadding.Substring(0, 1 + nDataSourcesLenMax - "Data Source Name".Length) + "Description");
                //  Put data source names and descriptions in listbox
                for (nIndex = 0; nIndex < nDataSources; nIndex++)
                    lbDataSourceNames.Items.Add(sl.ElementAt(nIndex).ToString());

                //  Need to padd out the function and information names
                sPadding = new String(' ', 32);
                //  Populate the functions listbox
                //  Put headings in listbox
                lbFunctions.Items.Add("Function" + sPadding.Substring(0, 31 - "Function".Length) + "Supported");
                //  Put functions in listbox
                int nFunctionsUsed = CODBC.FunctionsUsed();
                for (nIndex = 0; nIndex < nFunctionsUsed; nIndex++)
                    if (!(CODBC.FunctionName(nIndex) == null))
                        lbFunctions.Items.Add(CODBC.FunctionName(nIndex) + sPadding.Substring(0, 31 - CODBC.FunctionName(nIndex).Length));

                //  Populate the information listbox
                //  Put headings in listbox
                lbInformation.Items.Add("Information Type" + sPadding.Substring(0, 31 - "Information Type".Length) + "Information");
                //  Put informatino names in listbox
                int nInformationUsed = CODBC.InformationUsed();
                for (nIndex = 1; nIndex <= nInformationUsed; nIndex++)
                    if (!(CODBC.InformationName(nIndex) == null))
                        lbInformation.Items.Add(CODBC.InformationName(nIndex) + sPadding.Substring(0,31 - CODBC.InformationName(nIndex).Length));
            }
        }
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            //  Release all of the data sources we connected to
            string sDataSource;
            for (int nIndex = 1; nIndex < lbDataSourceNames.Items.Count; nIndex++)
            {
                sDataSource = (string)lbDataSourceNames.Items[nIndex];
                ODBCSources.Remove(sDataSource.Substring(0, nDataSourcesLenMax).Trim());
            }
            //  Release our reference class
            CODBCReferences = null;
            //  Exit the application
            Application.Current.Shutdown();
        }
    }
}
